<?php

/**
 * SQL Solution's ODBC specific code.
 *
 * <p>This is part of the SQL Solution.  See the sql-common.inc file
 * for more information, the license, etc.</p>
 *
 * @package    SQLSolution
 * @author     Daniel Convissor <danielc@analysisandsolutions.com>
 * @copyright  The Analysis and Solutions Company, 2001-2009
 * @version    $Name: rel-6-6 $ $Id: sql-od.inc,v 5.16 2009-12-24 23:35:22 danielc Exp $
 * @link       http://www.analysisandsolutions.com/software/sql/sql.htm
 * @see        sql-common.inc
 */

/**
 * SQL Solution's ODBC specific methods.
 *
 * @package    SQLSolution
 * @author     Daniel Convissor <danielc@analysisandsolutions.com>
 * @copyright  The Analysis and Solutions Company, 2001-2009
 * @version    $Name: rel-6-6 $
 * @link       http://www.analysisandsolutions.com/software/sql/sql.htm
 */
class SQLSolution_ODBCSpecifics extends SQLSolution_Customizations {

    /*
     * C O N N E C T I O N      S E C T I O N
     */

    /**
     *
     *
     * @param  string   $FileName  the file which called this method
     * @param  integer  $FileLine  the line number where this method was called
     *
     * @link   http://www.SqlSolution.info/sql-man.htm#Connect
     */
    function Connect($FileName, $FileLine) {
        ini_set('track_errors', 1);
        $php_errormsg = '';

        $this->SQLConnection = @odbc_connect($this->SQLDSN,
                                             $this->SQLUser,
                                             $this->SQLPassword,
                                             $this->SQLCursor)
                or die
                ($this->KillQuery($FileName, $FileLine, $php_errormsg));
    }

    /**
     *
     *
     * @param  string   $FileName  the file which called this method
     * @param  integer  $FileLine  the line number where this method was called
     *
     * @link   http://www.SqlSolution.info/sql-man.htm#PersistentConnect
     */
    function PersistentConnect($FileName, $FileLine) {
        ini_set('track_errors', 1);
        $php_errormsg = '';

        $this->SQLConnection = @odbc_pconnect($this->SQLDSN,
                                              $this->SQLUser,
                                              $this->SQLPassword,
                                              $this->SQLCursor)
                or die
                ($this->KillQuery($FileName, $FileLine, $php_errormsg));
    }

    /**
     *
     *
     * @param  string   $FileName  the file which called this method
     * @param  integer  $FileLine  the line number where this method was called
     *
     * @link   http://www.SqlSolution.info/sql-man.htm#ObtainHandle
     */
    function ObtainHandle($FileName, $FileLine) {
        $this->Connect($FileName, $FileLine);
    }

    /**
     *
     *
     * @param  string   $FileName  the file which called this method
     * @param  integer  $FileLine  the line number where this method was called
     *
     * @link   http://www.SqlSolution.info/sql-man.htm#Disconnect
     */
    function Disconnect($FileName, $FileLine) {
        @odbc_close($this->SQLConnection);
    }


    /*
     * Q U E R Y      S E C T I O N
     */


    /**
     *
     *
     * @param  string   $FileName  the file which called this method
     * @param  integer  $FileLine  the line number where this method was called
     *
     * @link   http://www.SqlSolution.info/sql-man.htm#RunQuery
     */
    function RunQuery($FileName, $FileLine) {
        if (!$this->SQLConnection) {
            $this->Connect($FileName, $FileLine);
        }

        $php_errormsg = '';

        if ($this->SQLRecordSet = @odbc_exec($this->SQLConnection,
                    $this->SQLQueryString))
        {
            $this->SQLRecordSetFieldCount =
                    @odbc_num_fields($this->SQLRecordSet);

            $Rows = @odbc_num_rows($this->SQLRecordSet);
            if ($Rows == -1) {
                /*
                 * The ODBC Driver being used is so stupid that it doesn't
                 * handle num_rows.  So, manually count the rows instead.
                 */
                if (@odbc_fetch_row($this->SQLRecordSet)) {
                    $Counter = 1;

                    while (@odbc_fetch_row($this->SQLRecordSet)) {
                        $Counter++;
                    }

                    @odbc_fetch_row($this->SQLRecordSet, 0);

                    $this->SQLRecordSetRowCount = $Counter;
                } else {
                    $this->SQLRecordSetRowCount = 0;
                }
            } else {
                $this->SQLRecordSetRowCount = $Rows;
            }
        } else {
            $this->KillQuery($FileName, $FileLine, $php_errormsg);
        }
    }

    /**
     *
     *
     * @param  string   $FileName  the file which called this method
     * @param  integer  $FileLine  the line number where this method was called
     *
     * @link   http://www.SqlSolution.info/sql-man.htm#RunQuery_RowsNeeded
     */
    function RunQuery_RowsNeeded($FileName, $FileLine, $RowsNeeded = 1) {
        $this->RunQuery($FileName, $FileLine);

        if ($this->SQLRecordSetRowCount == $RowsNeeded) {
            return 1;
        }
    }

    /**
     *
     *
     * @param  string   $FileName  the file which called this method
     * @param  integer  $FileLine  the line number where this method was called
     *
     * @link   http://www.SqlSolution.info/sql-man.htm#RunQuery_NoDuplicates
     */
    function RunQuery_NoDuplicates($FileName, $FileLine) {
        if (!$this->SQLConnection) {
            $this->Connect($FileName, $FileLine);
        }

        $php_errormsg = '';

        if ($this->SQLRecordSet = @odbc_exec($this->SQLConnection,
                    $this->SQLQueryString))
        {
            $this->SQLRecordSetFieldCount = 0;
            $this->SQLRecordSetRowCount = 0;
            return 1;
        } elseif (ereg('would create duplicate values', $php_errormsg)) {
            // Couldn't insert/update record due to duplicate key.
        } else {
            $this->KillQuery($FileName, $FileLine, $php_errormsg);
        }
    }

    /**
     *
     *
     * @param  string   $FileName  the file which called this method
     * @param  integer  $FileLine  the line number where this method was called
     *
     * @link   http://www.SqlSolution.info/sql-man.htm#ReleaseRecordSet
     */
    function ReleaseRecordSet($FileName, $FileLine) {
        @odbc_free_result($this->SQLRecordSet);
    }


    /*
     * F I E L D      D E F I N I T I O N S      S E C T I O N
     */


    /**
     *
     *
     * @param  string   $FileName  the file which called this method
     * @param  integer  $FileLine  the line number where this method was called
     *
     * @link   http://www.SqlSolution.info/sql-man.htm#FieldName
     */
    function FieldName($FileName, $FileLine, $FieldNumber) {
        $php_errormsg = '';

        if ($Output = @odbc_field_name($this->SQLRecordSet, $FieldNumber + 1)) {
            return $Output;
        } else {
            $this->KillQuery($FileName, $FileLine, $php_errormsg);
        }
    }

    /**
     *
     *
     * @param  string   $FileName  the file which called this method
     * @param  integer  $FileLine  the line number where this method was called
     *
     * @link   http://www.SqlSolution.info/sql-man.htm#FieldType
     */
    function FieldType($FileName, $FileLine, $FieldNumber) {
        $php_errormsg = '';

        if ($Output = @odbc_field_type($this->SQLRecordSet, $FieldNumber + 1)) {
            return $Output;
        } else {
            $this->KillQuery($FileName, $FileLine, $php_errormsg);
        }
    }

    /**
     *
     *
     * @param  string   $FileName  the file which called this method
     * @param  integer  $FileLine  the line number where this method was called
     *
     * @link   http://www.SqlSolution.info/sql-man.htm#FieldLength
     */
    function FieldLength($FileName, $FileLine, $FieldNumber) {
        $php_errormsg = '';

        if ($Output = @odbc_field_len($this->SQLRecordSet, $FieldNumber + 1)) {
            return $Output;
        } else {
            $this->KillQuery($FileName, $FileLine, $php_errormsg);
        }
    }


    /*
     * R E C O R D      D A T A      S E C T I O N
     */


    /**
     *
     *
     * @param  string   $FileName  the file which called this method
     * @param  integer  $FileLine  the line number where this method was called
     * @param  array    $SkipSafeMarkup  an array of field names to not parse
     *                                   safe markup on
     *
     * @link   http://www.SqlSolution.info/sql-man.htm#RecordAsAssocArray
     */
    function RecordAsAssocArray($FileName, $FileLine, $SkipSafeMarkup = array()) {
        $php_errormsg = '';

        /*
         * Sillyness neeeded to handle changes in PHP
         * while avoiding Warning messages.
         */
        if (phpversion() < '4.2.0') {
            $Row = '';
            $Code  = 'if (!@odbc_fetch_into($this->SQLRecordSet, $Row, &$Temp) && $php_errormsg != "") {';
            $Code .= '    $this->KillQuery($FileName, $FileLine, $php_errormsg);';
            $Code .= '}';
            eval($Code);
        } else {
            if (!@odbc_fetch_into($this->SQLRecordSet, $Temp) AND $php_errormsg != '') {
                $this->KillQuery($FileName, $FileLine, $php_errormsg);
            }
        }

        foreach ($Temp as $Key => $Val) {
            if ($this->SQLEscapeHTML != 'N') {
                $Val = htmlspecialchars($Val);
            }
            if ($this->SQLSafeMarkup == 'Y') {
                if (!in_array($Key, $SkipSafeMarkup)) {
                    $Val = $this->ParseSafeMarkup($Val);
                }
            }
            $FieldName = $this->FieldName('FieldName() had error when RAAA() '
                    . "was called by $FileName", "$FileLine", $Key);
            $Output[$FieldName] = $Val;
        }

        if (isset($Output)) {
            return $Output;
        }
    }

    /**
     *
     *
     * @param  string   $FileName  the file which called this method
     * @param  integer  $FileLine  the line number where this method was called
     * @param  array    $SkipSafeMarkup  an array of field numbers to not parse
     *                                   safe markup on
     *
     * @link   http://www.SqlSolution.info/sql-man.htm#RecordAsEnumArray
     */
    function RecordAsEnumArray($FileName, $FileLine, $SkipSafeMarkup = array()) {
        $php_errormsg = '';

        /*
         * Sillyness neeeded to handle changes in PHP
         * while avoiding Warning messages.
         */
        if (phpversion() < '4.2.0') {
            $Row = '';
            $Code  = 'if (!@odbc_fetch_into($this->SQLRecordSet, $Row, &$Temp) && $php_errormsg != "") {';
            $Code .= '    $this->KillQuery($FileName, $FileLine, $php_errormsg);';
            $Code .= '}';
            eval($Code);
        } else {
            if (!@odbc_fetch_into($this->SQLRecordSet, $Temp) AND $php_errormsg != '') {
                $this->KillQuery($FileName, $FileLine, $php_errormsg);
            }
        }

        foreach ($Temp as $Key => $Val) {
            if ($this->SQLEscapeHTML != 'N') {
                $Val = htmlspecialchars($Val);
            }
            if ($this->SQLSafeMarkup == 'Y') {
                if (!in_array($Key, $SkipSafeMarkup)) {
                    $Val = $this->ParseSafeMarkup($Val);
                }
            }
            $Output[] = $Val;
        }

        if (isset($Output)) {
            return $Output;
        }
    }

    /**
     *
     *
     * @param  string   $FileName  the file which called this method
     * @param  integer  $FileLine  the line number where this method was called
     *
     * @link   http://www.SqlSolution.info/sql-man.htm#InsertID
     */
    function InsertID($FileName, $FileLine, $Table = '', $Field = '', $Where = '',
                      $Sequence = '') {
        $this->SQLQueryString = "SELECT $Field FROM $Table WHERE $Where";
        $this->RunQuery(__FILE__,__LINE__);
        if ($this->SQLRecordSetRowCount != 1) {
            $this->KillQuery($FileName, $FileLine,
                'The terms specified returned either no Insert ID or too '
                . "many Insert ID's.");
        }
        list($Val) = $this->RecordAsEnumArray(__FILE__,__LINE__);
        return $Val;
    }

    /**
     *
     *
     * @param  string   $FileName  the file which called this method
     * @param  integer  $FileLine  the line number where this method was called
     *
     * @link   http://www.SqlSolution.info/sql-man.htm#GoToRecord
     */
    function GoToRecord($FileName, $FileLine, $Row = 0) {
        @odbc_fetch_row($this->SQLRecordSet, $Row);
    }

    /**
     * Makes input safe for use as a value in queries
     *
     * Surrounds the string with quote marks.  If the value is NULL, change it
     * to the unquoted string "NULL".
     *
     * @param  string   $FileName  the file which called this method
     * @param  integer  $FileLine  the line number where this method was called
     * @param  mixed    $Value     the value to be escaped
     *
     * @return string  the escaped string
     */
    function Escape($FileName, $FileLine, $Value) {
        if ($Value === null) {
            return 'NULL';
        } else {
            return "'" . str_replace("'", "''", $Value) . "'";
        }
    }
}
